import java.net.*;
import java.io.IOException;

public class SmartURL {

	private URL url;

	public SmartURL(URL u) {
		url = u;
	}

	public SmartURL(String u) throws MalformedURLException {
		url = new URL(u);
	}

	public SmartURL(SmartURL u1, String u2) throws MalformedURLException {
		
		//System.out.println("SMARTURL with " + u1 + " and " + u2);

		String u;
			
		//avoid null u2
		if (u2.equals("") || u2 == null)
			u = u1.toString();
		else {

			u = u1.getProtocol() + "://";
			u = u + u1.getHost();
			if (u1.getPort() != -1)
				u = u + ":" + u1.getPort();

			u = u + "/";

			//are we dealing with a absolute URL ?
			if (u2.charAt(0) == '/') 
				u = u + u2.substring(1);
			else 
				//are we dealing with a complete URL ?
				if (u2.toLowerCase().startsWith("http://") ||
					u2.toLowerCase().startsWith("ftp://") ||
					u2.toLowerCase().startsWith("mailto:")) 
					u = u2;
				else {
					//must be a relative URL
					u += u1.getPath();
					if (u.charAt(u.length()-1) != '/')
						u += "/";
					u += u2;
				}
		}

		//System.out.println("SMARTURL is " + u); 
		url = new URL(u);
	}

	//return only the file name without the path and the bookmark
	public String getFile() {

		String u;
		int    pos;

		u = url.getFile();

		if (u.indexOf(".") != -1) {
			//remove the bookmark
			pos = u.lastIndexOf('#');
			if (pos != -1) {
				u = u.substring(0, pos);
			}
	
			//extract the filename
			pos = u.lastIndexOf('/');
			if (pos != -1) {
				u = u.substring(pos+1);
			}
		}
		else 
			u = "";

		return u;
	}

	public String getFileType() {

		String u   = getFile();
		int    pos;

		pos = u.lastIndexOf('.');

		if (pos != -1)
			u   = u.substring(pos+1);

		return u;
	}

	//return only the path of a URL
	public String getPath() {

		String u;
		int    pos;

		u = url.getFile();

		//System.out.println("GETPATH U1 IS " + u);

		//is there a filename ?
		if (u.indexOf(".") != -1) {
			//remove the file and bookmark name
			pos = u.lastIndexOf('/');
			if (pos != -1) {
				u = u.substring(0, pos);
			}
		}

		//find the first slash after the hostname
		pos = u.indexOf(url.getHost());
		pos = u.indexOf('/', pos+1);

		u = u.substring(pos+1);
	
		return u;		
	}

	public boolean isImage() {
		boolean ret;

		if (getFileType().equalsIgnoreCase("JPG") ||
			getFileType().equalsIgnoreCase("GIF"))
			ret = true;
		else
			ret = false;

		return ret;
	}

	public boolean isSameHost(SmartURL u) {
		return url.getHost().equalsIgnoreCase(u.getHost());
	}

	public String getHost() {
		return url.getHost();
	}

	public String getProtocol() {
		return url.getProtocol();
	}

	public int getPort() {
		return url.getPort();
	}

	public URLConnection openConnection() throws IOException {
		return url.openConnection();
	}

	public String toString() {
		return url.toString();
	}

