import java.net.*;
import java.io.*;
import java.util.*;

public class URLExplorer {
	

	private String   filename    = "default.htm";
	private String   defaultPath;
	private SmartURL startURL;

	
	Hashtable visitedLinks;
	Hashtable visitedImages;

	public URLExplorer(String url, int iDepth, String defPath) {

		visitedLinks  = new Hashtable();
		visitedImages = new Hashtable();

		startURL    = urlFromString(url);
		defaultPath = defPath;

		explore(startURL, 1, iDepth);
	}


	public void explore(SmartURL url, int nDeep, int nMaxDeep) {
	
		//connection related variables
		URLConnection  uc;
		BufferedReader in;
		String         inputLine, outputLine, path;

		//to write the file
		FileWriter     fw;

		//are we still on the same site ?
		if (! url.isSameHost(startURL)) {
			System.out.println("NOT SAME HOST : " + url);
			return;
		}
		
		//have we already visited this URL ?
		if (visitedLinks.get(url.toString())==null)
			//no, add it to our list
			visitedLinks.put(url.toString(), url);
		else {
			//yes, leave it as is
			System.out.println("ALREADY VISITED : " + url);
			return;
		}

		//to store the url and img name founded
		Vector links = new Vector();
		Vector lineLinks;        
			
		//display feedback about exploration
		printLocation(url, nDeep);
		
		try {
			uc = url.openConnection();
			in = new BufferedReader(new InputStreamReader(uc.getInputStream()));
			
			//create the dirs path and filename
			path = createDir(url);
			
			if (url.getFile().equals(""))
				path += "/" + filename;
			else {
				if (url.getFile().charAt(0) != '/')
					path += "/";
				path += url.getFile();
			}				

			//convert URL specific syntax to file syntax
			path = normalize(path);

			fw = new FileWriter(path);
		
			while ((inputLine = in.readLine()) != null) {
				lineLinks = parseLine(links, inputLine);
				
				//if there is at least one ref, replace it
				if (lineLinks.isEmpty())
					outputLine = inputLine;
				else
					outputLine = replaceLinks(lineLinks, inputLine, url);
				
				//modify the first line to change the URL's base
				outputLine = addBaseTag(outputLine);
				
				fw.write(outputLine + "\r\n");
			}

			in.close();      
			fw.close();

			//dump the vector
			for (Enumeration e = links.elements() ; e.hasMoreElements() ;) {
				String el = (String)e.nextElement();

				//if we are not yet at the end of the depth
				//nor the link is internal to the doc itself (bookmark)
				if (!el.startsWith("#")) {
					SmartURL newURL = new SmartURL (url, el);
					if (newURL.isImage())
						saveImageFile(newURL);
					else if (nDeep<=nMaxDeep) 
						explore(newURL, nDeep+1, nMaxDeep);					
				}
			}	
		}
		catch(FileNotFoundException e) {
			stdExceptionHandling(e);
		}
		catch(IOException e) {
			stdExceptionHandling(e);
			System.exit(0);
		}
	}


	private Vector parseLine(Vector v, String line) {
		
		boolean found = false;
		StreamTokenizer st;
		Vector          lineLinks = new Vector();

		try {
		
			//prepare the string tokenizer
			st = new StreamTokenizer(new StringReader(line));
    
			//adapt it for HTML
			st.quoteChar('"');
			st.ordinaryChar('/'); //Ok, not really needed but...
			
			//take the first token of the line
			st.nextToken();

			while (st.ttype != st.TT_EOF) {

				//look for the sequence IMG SRC or FRAME SRC
				//when found, extract the filename that follows
				if (st.ttype == st.TT_WORD)
					if (st.sval.equalsIgnoreCase("IMG") ||
					    st.sval.equalsIgnoreCase("FRAME")) {
						while (!found && st.ttype != st.TT_EOF) {
							st.nextToken();
							if (st.ttype == st.TT_WORD)
								if (st.sval.equalsIgnoreCase("SRC")) {
									//skip the equal sign
									st.nextToken();
									st.nextToken();
									if (st.ttype == '"') {
										addFilename(v, st.sval);
										addFilename(lineLinks, st.sval);
										found = true;
									}
								}
						}
					}

				//look for the BODY BACKGROUND tag
				if (st.ttype == st.TT_WORD)
					if (st.sval.equalsIgnoreCase("BODY")) {
						st.nextToken();
						if (st.ttype == st.TT_WORD)
							if (st.sval.equalsIgnoreCase("BACKGROUND")) {
								//skip the equal sign
								st.nextToken();
								st.nextToken();
								if (st.ttype == '"') {
									addFilename(v, st.sval);
									addFilename(lineLinks, st.sval);
								}
							}
					}

				//look for the sequence HREF
				//when found, extract the following filename
				if (st.ttype == st.TT_WORD)
					if (st.sval.equalsIgnoreCase("HREF")) {
						//skip the equal sign
						st.nextToken();
						st.nextToken();
						if (st.ttype == '"') {
							addFilename(v, st.sval);
							addFilename(lineLinks, st.sval);
						}
					}
				
				//get the next token of the line
				st.nextToken();
			}
		}
		catch(IOException e) {
			stdExceptionHandling(e);
		}

		return lineLinks;
	}

	private String addFilename(Vector v, String f) {
		v.addElement(f);
		return f;
	}

	private SmartURL urlFromString(String u1) {
		
		try {
			return new SmartURL(u1);      
		}
		catch(MalformedURLException e) {
			stdExceptionHandling(e);
			System.exit(0);
			return null;
		}
	}

	private void stdExceptionHandling(Exception e) {
	  System.out.println("Error : " + e.getMessage());
	  e.printStackTrace();
	}

	private String createDir(SmartURL url) {

		String          ret;
		String          dirToCreate;
		StringTokenizer st;

		try {
			ret = defaultPath + url.getPath();
			if (ret.charAt(ret.length()-1) != '\\')
				ret += "\\";

			st = new StringTokenizer(ret, "\\");
    
			//take the leading drive_letter:
			dirToCreate = st.nextToken();
			while (st.hasMoreTokens()) {
				dirToCreate += "\\" + st.nextToken();

				File f = new File(dirToCreate);
				f.mkdirs();

			}
			ret = dirToCreate;
		}
		catch(NullPointerException e) {
			stdExceptionHandling(e);
			ret = "";
		}

		return ret;
	}

	private String replaceLinks(Vector v, String l, SmartURL u) {

		int      s;
		SmartURL newURL;
		String   newLine = l;
		
		//for debugging
		String   old = l;

		for (Enumeration e = v.elements() ; e.hasMoreElements() ;) {

			String el = (String)e.nextElement();
			
			try {
				newURL = new SmartURL (u, el);
				
				//if the new URL is on the same host
				//replace the URL by the new relative position
				if (newURL.isSameHost(startURL) &&
					! newURL.toString().startsWith("mailto:") && 
					! newURL.toString().startsWith("ftp://")) {

					s = l.indexOf(el);
					newLine =  l.substring(0, s);
					newLine += newURL.getPath() + "/" + newURL.getFile();
					newLine += l.substring(s + el.length());
				}
			}
			catch(MalformedURLException ex) {
				stdExceptionHandling(ex);
				System.exit(0);
			}
			catch(StringIndexOutOfBoundsException ex) {
				newLine = l;
			}

			l = newLine;
			
		} //for
		
		//System.out.println("REPLACE OLD IS " + old);
		//System.out.println("REPLACE NEW IS " + newLine);
		return newLine;
	}

	private void saveImageFile(SmartURL u) {

		String            path;
		ImageRetriever    ir;

		//have we already visited this image ?
		if (visitedImages.get(u.toString())==null)
			//no, add it to our list
			visitedImages.put(u.toString(), u);
		else {
			//yes, leave it as is
			System.out.println("ALREADY GET : " + u);
			return;
		}

		//create the dirs path and filename
		path = createDir(u);
		
		if (u.getFile().charAt(0) != '/')
			path += "/";
		path += u.getFile();

		path = normalize(path);

		//span a new thread to retrieve the image
		System.out.println("GET IMAGE = " + u);
		ir = new ImageRetriever(path, u);
		ir.start();
	}

	public String addBaseTag(String l) {
		if (l.toLowerCase().indexOf("<html>") != -1)
			l += "<base href=" + '"' + defaultPath + '"' + ">";
		return l;
	}

	public String normalize(String p) {
		int    pos;
		String ret;
		
		while ((pos = p.indexOf("%20")) != -1) {
			ret = p.substring(0, pos);
			ret += " ";
			ret += p.substring(pos + 3);
			p = ret;
		}

		return p;
	}

	private void printLocation(SmartURL u, int d) {
		for (int i = 0; i<=d; i++)
			System.out.print(" ");
		System.out.println("Exploring : " + u);
	}
